@@ -6,14 +6,16 @@ use Carp;
 use Time::HiRes;
 use Config;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our $DEBUG = $ENV{FQ_DEBUG} || 0;
 our $DEFAULT_IMPL;
 
 our $NOTIFY_OK = $ENV{FQ_NOTIFY} // do {
     $Config::Config{sig_name} =~ /\bIO\b/;
+    0; # disable
 };
-our $SLEEP_INTERVAL = $NOTIFY_OK ? 10 : 1;
+our $SLEEP_INTERVAL = $NOTIFY_OK ? 2 : 1;
+our $SLEEP_INTERVALX = 1;
 
 sub new {
     my ($pkg,%opts) = @_;
@@ -176,7 +178,7 @@ Forks::Queue - queue that can be shared across processes
 
 =head1 VERSION
 
-0.04
+0.05
 
 =head1 SYNOPSIS
 
@@ -198,7 +200,8 @@ Forks::Queue - queue that can be shared across processes
 
 =head1 DESCRIPTION
 
-Interface for a queue object that can be shared across processes.
+Interface for a queue object that can be shared across processes
+and threads. 
 Available implementations are L<Forks::Queue::File|Forks::Queue::File>,
 L<Forks::Queue::Shmem|Forks::Queue::Shmem>,
 L<Forks::Queue::SQLite|Forks::Queue::SQLite>.
@@ -215,7 +218,7 @@ This will include scalars and most unblessed references
   [1,2,3,"forty-two"]
   { name=>"a job", timestamp=>time, input=>{foo=>[19],bar=>\%bardata} }
 
-but will generally prohibit data with blessed references and code references
+but will generally preclude data with blessed references and code references
 
   { name => "bad job", callback => \&my_callback_routine }
   [ 46, $url13, File::Temp->new ]
@@ -233,6 +236,11 @@ If the C<impl> option is not provided, this call will fail.
 
 Options that should be supported on all implementations include
 
+=cut
+
+# not documented yet: set $Forks::Queue::DEFAULT_IMPL to set
+# implementation to be used whenever  impl => ...  is not specified
+    
 =over 4
 
 =item * C<< style => 'fifo' | 'lifo' >>
@@ -358,7 +366,7 @@ its queue size limit (as determined by the L<"limit"> attribute),
 this method will either block until queue capacity is available,
 or issue a warning about the uninserted items and return the
 number of items added, depending on the queue's setting for
-L<"on_limit">.
+L<"on_limit"|Forks::Queue/"new">.
 
 
 =head2 unshift
@@ -578,7 +586,7 @@ the list would cause the queue size to exceed the limit, this
 method will either block until capacity to insert the whole list
 becomes available, or it will insert items up to the queue size
 limit and issue a warning about the uninserted items, depending
-on the queue's L<"on_limit"> setting.
+on the queue's L<"on_limit"|Forks::Queue/"new"> setting.
 
 This method is inefficient for some queue implementations.
 
@@ -693,6 +701,7 @@ See http://dev.perl.org/licenses/ for more information.
 #
 #     priorities
 #     Directory implementation (see Queue::Dir)
-# _X_ use SIGIO to wait for tasks/capacity, not just sleep
-#         pros: better resolution, response than sleep 1
-#         cons: POSIX only, handled simultaneously by all queues in a process,
+# _X_ Better thread support
+#     network implementation with simple client-server
+#     even better thread support, 2nd signal from main to threads    
+#     import function to set global impl, limit, on_limit settings
