@@ -8,7 +8,7 @@ use Time::HiRes;
 use base 'Forks::Queue';
 use 5.010;    #  using  // //=  operators
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
 
@@ -76,6 +76,10 @@ sub _SYNCWA (&$) {    # wantarray version of _SYNC
     return @result;
 }
 
+sub _PID {
+    $INC{'threads.pm'} ? join("-", $$, threads->tid) : $$
+}
+
 sub new {
     my ($class, %opts) = @_;
     $opts{file} //= _impute_file();
@@ -93,7 +97,7 @@ sub new {
     $opts{_tell} = $opts{_header_size};        # file position of cursor
 
     $opts{_count} = 0;          # index of next item to be appended
-    $opts{_pids} = { $$ => 'P' };
+    $opts{_pids} = { _PID() => 'P' };
     $opts{_version} = $Forks::Queue::VERSION;
 
     # how often to refactor the queue file. use small values to keep file
@@ -150,7 +154,7 @@ sub DESTROY {
         $self->_read_header;
         $DEBUG and print STDERR "$$ DESTROY: pids at destruction: ",
                                 join(" ",keys %{$self->{_pids}}),"\n";
-        delete $self->{_pids}{$$};
+        delete $self->{_pids}{_PID()};
         $self->_write_header;
     } $self;
     };
@@ -231,9 +235,45 @@ sub _notify {
 
     my $self = CORE::shift;
     _SYNC { $self->_read_header } $self;
-    my @pids = grep { $_ != $$ } keys %{$self->{_pids}};
+    my @ids = keys %{$self->{_pids}};
+    my (@pids,@tids);
+    my $me = _PID();
+    $DEBUG && print STDERR "_notify \$me=$me  \@ids=@ids\n";
+    foreach my $id (@ids) {
+        my ($p,$t) = split /-/,$id;
+        if (!$p) {
+            ($p,$t) = (-$t,0);
+        }
+        if ($p != $$) {
+            push @pids, $p;
+        } elsif (defined($t) && $id ne $me) {
+            push @tids, $t;
+        }
+    }
+    if (@tids) {
+        $DEBUG && print STDERR "$$ notify: tid @tids\n";
+        foreach my $tid (@tids) {
+            my $thr = threads->object($tid);
+            if ($thr) {
+                my $z7;
+                $thr && ($z7 = $thr->kill('IO')) &&
+                    $DEBUG && print STDERR "_notify to tid $$-$tid \$z7=$z7\n";
+                if ($tid ne $tids[-1]) {
+                    #Time::HiRes::sleep 0.25;
+                }
+
+                # $thr->kill is not reliable?
+                
+            } elsif ($tid == 0) {
+                $DEBUG && print STDERR "_notify SIGIO to tid main\n";
+                kill 'IO', $$;
+            } else {
+                $DEBUG && print STDERR "_notify failed to SIGIO tid $tid\n";
+            }
+        }
+    }
     if (@pids) {
-        $DEBUG && print STDERR "$$ notify: @pids\n";
+        $DEBUG && print STDERR "_notify to pids @pids\n";
         kill 'IO', @pids;
     }
 }
@@ -260,10 +300,11 @@ sub end {
             carp "Forks::Queue: end() called from $$, ",
                  "previously called from $self->{_end}";
         } else {
-            $self->{_end} = $$;
+            $self->{_end} = _PID();
         }
         $self->_write_header;
     } $self;
+    $self->_notify;
     return;
 }
 
@@ -278,18 +319,27 @@ sub status {
 
 sub _check_pid {
     my ($self) = @_;
-    if (!defined $self->{_pids}{$$}) {
+    if (!defined $self->{_pids}{_PID()}) {
+        if ($Forks::Queue::NOTIFY_OK) {
+            if (_PID() =~ /.-[1-9]/) {
+                # SIGIO can't be reliably passed to threads, so can't
+                # rely on long sleep command being interrupted
+                $Forks::Queue::SLEEP_INTERVAL = 1;
+            }
+            $SIG{IO} = sub { };
+        }
         open $self->{_fh}, '+<', $self->{file};
         if ($self->{_locked}) {
-            $DEBUG && print STDERR "Forks::Queue: $$ new pid update header\n";
-            $self->{_pids}{$$} = 'C';
+            $DEBUG && print STDERR
+                "Forks::Queue: $$ new pid update header\n";
+            $self->{_pids}{_PID()} = 'C';
             $self->_write_header;
             return;
         } else {
             $DEBUG and print STDERR "Forks::Queue: $$ new pid sync\n";
             _SYNC {
                 $self->_read_header;
-                $self->{_pids}{$$} = 'C';
+                $self->{_pids}{_PID()} = 'C';
                 $self->_write_header;
             } $self;
             return;
@@ -377,6 +427,9 @@ sub push {
         }
         $self->_write_header;
     } $self;
+    if ($pushed && $DEBUG) {
+        print STDERR "_notify from push(\$pushed=$pushed)\n";
+    }
     $self->_notify if $pushed;
 
     if ($failed_items) {
@@ -398,13 +451,21 @@ sub unshift {
     return $self->insert(0, @items);
 }
 
+sub _SLEEP {
+    my $self = shift;
+    # my $tid = threads->self;
+    my $n = sleep($Forks::Queue::SLEEP_INTERVAL || 1);
+    Carp::cluck("LONG SLEEP \$n=$n") if $n > 10;
+    return $n;
+}
+
 sub _wait_for_item {
     my ($self) = @_;
     my $ready = 0;
     do {
         _SYNC { $self->_read_header } $self;
         $ready = $self->{_avail} || $self->{_end} || $self->_expired;
-        sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
+        _SLEEP($self) if !$ready;# sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
     } while !$ready;
     return $self->{_avail};
 }
@@ -418,7 +479,7 @@ sub _wait_for_capacity {
         } else {
             _SYNC { $self->_read_header } $self;
             $ready = $self->{_avail} < $self->{limit} && !$self->{_end};
-            sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
+            _SLEEP() if !$ready; #sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
         }
     } while !$ready;
     return $self->{_avail} < $self->{limit};
@@ -456,7 +517,7 @@ sub _dequeue_back {
             $self->_write_header;
         } $self;
         last if @return || $self->{_end} || $self->_expired;
-        sleep($Forks::Queue::SLEEP_INTERVAL || 1);
+        _SLEEP(); #sleep($Forks::Queue::SLEEP_INTERVAL || 1);
     }
     $self->_notify if @return;
     if ($self->_expired && @return == 0) {
@@ -502,7 +563,7 @@ sub _dequeue_front {
             $self->_write_header;
         } $self;
         last if @return || $self->{_end} || $self->_expired;
-        sleep($Forks::Queue::SLEEP_INTERVAL || 1);
+        _SLEEP(); #sleep($Forks::Queue::SLEEP_INTERVAL || 1);
     }
     $self->_notify if @return;
     if ($self->_expired && @return == 0) {
@@ -816,7 +877,7 @@ sub pop {
         do {
             _SYNC { $self->_read_header } $self;
         } while (!$self->{_avail} && !$self->{_end} && 
-                 1 + sleep($Forks::Queue::SLEEP_INTERVAL || 1));
+                 1 + _SLEEP()); #sleep($Forks::Queue::SLEEP_INTERVAL || 1));
 
         return if $self->{_end} && !$self->{_avail};
 
@@ -965,19 +1026,24 @@ sub _impute_file {
     $base =~ s{.*[/\\](.)}{$1};
     $base =~ s{[/\\]$}{};
     $id++;
-    if ($^O eq 'MSWin32') {
-        return "C:/Temp/fq-$$-$id-$base";
-    }
     my $file;
+    my @candidates;
+    if ($^O eq 'MSWin32') {
+        @candidates = (qw(C:/Temp C:/Windows/Temp));
+    } else {
+        @candidates = qw(/tmp /var/tmp);
+    }
 
     # try hard to avoid using an NFS drive
-    for my $candidate ($ENV{FORKS_QUEUE_DIR}, "/tmp", "/var/tmp",
+    for my $candidate ($ENV{FORKS_QUEUE_DIR}, $ENV{TMPDIR}, $ENV{TEMP},
+                       $ENV{TMP}, @candidates,
                        $ENV{HOME}, ".") {
         next if !defined($candidate);
-        next if ! -d $candidate;
-        $file //= "$candidate/.fq-$$-$id-base";
-        next if __is_nfs($candidate);
-        return "$candidate/.fq-$$-$id-$base";
+        if (-d $candidate && -w _ && -x _) {
+            $file //= "$candidate/.fq-$$-$id-base";
+            next if __is_nfs($candidate);
+            return "$candidate/.fq-$$-$id-$base";
+        }
     }
 
     carp "Forks::Queue::File: queue file $file might be on an NFS filesystem!";
@@ -992,7 +1058,7 @@ Forks::Queue::File - file-based implementation of Forks::Queue
 
 =head1 VERSION
 
-0.04
+0.05
 
 =head1 SYNOPSIS
 
