@@ -9,7 +9,7 @@ use Time::HiRes 'time';
 use base 'Forks::Queue';
 use 5.010;    #  using  // //=  operators
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
 
@@ -37,7 +37,7 @@ sub new {
     }
 
     my $exists = $opts{join} && -f $opts{db_file};
-    $opts{_pid} = $$;
+    $opts{_pid} = [ $$, TID() ];
     # process id is tied to database handle. If process id doesn't match
     # $self->{_pid}, we must open a new process id.
 
@@ -76,14 +76,14 @@ sub _init {
         return;
     }
 
-    my $z2 = $dbh->do("CREATE TABLE pids (pid mediumint)");
+    my $z2 = $dbh->do("CREATE TABLE pids (pid mediumint,tid mediumint)");
     if (!$z2) {
         carp "Forks::Queue::SQLite: error creating init table";
         return;
     }
 
-    my $sth = $dbh->prepare("INSERT INTO pids VALUES (?)");
-    my $z3 = $sth->execute($self->{_pid});
+    my $sth = $dbh->prepare("INSERT INTO pids VALUES (?,?)");
+    my $z3 = $sth->execute(@{$self->{_pid}});
     if (!$z3) {
         carp "Forks::Queue::SQLite: error adding process id to tracker";
         return;
@@ -96,26 +96,30 @@ sub _init {
     }
 
     $self->_status("db_file", $self->{db_file});
-    $self->_status("owner", $self->{_pid});
+    $self->_status("owner", "@{$self->{_pid}}");
     $self->_status("style", $self->{style});
     $self->_status("limit", $self->{limit});
     $self->_status("on_limit", $self->{on_limit});
     return 1;
 }
 
+sub TID { $INC{'threads.pm'} ? threads->tid : 0 }
+
 sub _dbh {
     my $self = CORE::shift;
-    if ($$ == $self->{_pid} && $self->{_dbh}) {
+    my $tid = TID();
+    if ($self->{_dbh} && $$ == $self->{_pid}[0] && $tid == $self->{_pid}[1]) {
         return $self->{_dbh};
     }
-    $self->{_dbh} = DBI->connect(
-        "dbi:SQLite:dbname=".$self->{db_file},"","");
+
+    $self->{_pid} = [$$,$tid];
+    $self->{_dbh} =
+        DBI->connect("dbi:SQLite:dbname=".$self->{db_file},"","");
     $self->{_dbh}{AutoCommit} = 1;
-    $self->{_pid} = $$;
 
     $self->{_dbh}->begin_work;
-    $self->{_dbh}->do("DELETE FROM pids WHERE pid=$$");
-    $self->{_dbh}->do("INSERT INTO pids VALUES ($$)");
+    $self->{_dbh}->do("DELETE FROM pids WHERE pid=$$ AND tid=$tid");
+    $self->{_dbh}->do("INSERT INTO pids VALUES ($$,$tid)");
     $self->{_dbh}->commit;
     $self->{style} = $self->_status("style");
     $self->{limit} = $self->_status("limit");
@@ -126,14 +130,19 @@ sub _dbh {
 sub DESTROY {
     my $self = CORE::shift;
     my $dbh = $self->_dbh;
-    $dbh->begin_work;
-    $dbh->do("DELETE FROM pids WHERE pid=$$");
-    my $sth = $dbh->prepare("SELECT COUNT(*) FROM pids");
-    my $z = $sth->execute;
-    my $t = $sth->fetchall_arrayref;
-    $dbh->commit;
-    $DEBUG and print STDERR "$$ DESTROY npids=$t->[0][0]\n";
-    $dbh->disconnect;
+    my $tid = $self->{_pid} ? $self->{_pid}[1] : TID();
+    my $t = [[0]];
+    my $pid_rm = $dbh && eval {
+        $dbh->begin_work;
+        $dbh->do("DELETE FROM pids WHERE pid=$$ AND tid=$tid");
+        my $sth = $dbh->prepare("SELECT COUNT(*) FROM pids");
+        my $z = $sth->execute;
+        $t = $sth->fetchall_arrayref;
+        $dbh->commit;
+        $DEBUG and print STDERR "$$ DESTROY npids=$t->[0][0]\n";
+	1;
+    };
+    $dbh && eval { $dbh->disconnect };
     if ($t->[0][0] == 0) {
         $DEBUG and print STDERR "$$ Unlinking files from here\n";
         unlink $self->{db_file} unless $self->{persist};
@@ -183,6 +192,7 @@ sub end {
         $self->_status("end",$$);
         $dbh->commit;
     }
+    $self->_notify;
     return;
 }
 
@@ -239,7 +249,7 @@ sub limit :lvalue {
 
 sub status {
     my $self = CORE::shift;
-    my $dbh = $self->{_dbh};
+    my $dbh = $self->_dbh;
     my $status = {};
     my $sth = $dbh->prepare("SELECT key,value FROM status");
     my $z = $sth->execute;
@@ -341,7 +351,7 @@ sub _wait_for_item {
     my $ready = 0;
     do {
         $ready = $self->_avail || $self->_end || $self->_expired;
-        sleep 1 if !$ready;
+        sleep($Forks::Queue::SLEEP_INTERVAL || 1) if !$ready;
     } while !$ready;
     return $self->{avail};
 }
@@ -355,7 +365,7 @@ sub _wait_for_capacity {
     while (!$ready) {
         last if $self->_avail < $self->{limit};
         last if $self->_end;
-        sleep 1;
+        sleep($Forks::Queue::SLEEP_INTERVAL || 1);
     }
     return $self->{avail} < $self->{limit};
 }
@@ -552,8 +562,15 @@ sub insert {
 sub _retrieve {
     my $self = CORE::shift;
     my $tfactor = CORE::shift;
+        # tfactor = -1: select newest items first
+        # tfactor = +1: select oldest items first
     my $purge = CORE::shift;
+        # purge = 0: do not delete items that we retrieve
+        # purge = 1: delete items that we retrieve
     my $block = CORE::shift;
+        # block = 0: no block if queue is empty
+        # block = 1: block only if queue is empty
+        # block = 2: block if full request can not be fulfilled
     my $lo = CORE::shift;
     my $hi = @_ ? $_[0] : $lo+1;
     return if $hi <= $lo;
@@ -625,8 +642,10 @@ sub _retrieve {
         }
         $dbh->commit;
     } continue {
-        if ($self->{_end} || $self->_expired) {
-            $block = 0;
+        if ($block) {
+            if ($self->_end || $self->_expired) {
+                $block = 0;
+            }
         }
     }
     return @_ ? @return : $return[0] // ();
@@ -715,14 +734,21 @@ sub _notify {
 
     my $self = CORE::shift;
     my $dbh = $self->_dbh;
-    my $sth = $dbh->prepare("SELECT pid FROM pids");
+    my $sth = $dbh->prepare("SELECT pid,tid FROM pids");
     my $z = $sth->execute;
     my $pt = $sth->fetchall_arrayref;
-    my @pids = grep { $_ != $$ } map { $_->[0] } @$pt;
+    my @pids = map { $_->[0] } grep { $_->[0] != $$ } @$pt;
     if (@pids) {
         $DEBUG && print STDERR "$$ notify: @pids\n";
         kill 'IO', @pids;
     }
+    my @tids = map { $_->[1] } grep { $_->[0] == $$ && $_->[1] != TID() } @$pt;
+    if (@tids) {
+        foreach my $tid (@tids) {
+            my $thr = threads->object($tid);
+            $thr && $thr->kill('IO');
+        }
+    }
 }
 
 my $id = 0;
@@ -731,11 +757,23 @@ sub _impute_file {
     $base =~ s{.*[/\\](.)}{$1};
     $base =~ s{[/\\]$}{};
     $id++;
+    my @candidates;
     if ($^O eq 'MSWin32') {
-        return "C:/Temp/fq-$$-$id-$base.sql3";
+        @candidates = (qw(C:/Temp C:/Windows/Temp));
     } else {
-        return "/tmp/.fq-$$-$id-$base.sql3";
+        @candidates = qw(/tmp /var/tmp);
     }
+    for my $candidate ($ENV{FORKS_QUEUE_DIR}, $ENV{TMPDIR}, $ENV{TEMP},
+                       $ENV{TMP}, @candidates,
+                       $ENV{HOME}, ".") {
+        if (defined($candidate) && $candidate ne '' &&
+            -d $candidate && -w _ && -x _) {
+            return $candidate . "/fq-$$-$id-$base.sql3";
+        }
+    }
+    my $file = "./fq-$$-$id-$base.sql3";
+    carp "Forks::Queue::SQLite: queue db file $file might not be a good location!";
+    return $file;
 }
 
 sub _DUMP {
@@ -774,7 +812,7 @@ Forks::Queue::SQLite - SQLite-based implementation of Forks::Queue
 
 =head1 VERSION
 
-0.04
+0.05
 
 =head1 SYNOPSIS
 
