@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 our $DEV_SHM = "/dev/shm";
 our $DEBUG;
 *DEBUG = \$Forks::Queue::DEBUG;
@@ -115,7 +115,7 @@ Forks::Queue::Shmem - Forks::Queue implementation using shared memory
 
 =head1 VERSION
 
-0.04
+0.05
 
 =head1 DESCRIPTION
 
@@ -132,7 +132,7 @@ See L<Forks::Queue> for the public API to this class.
 =head2 Constructor options
 
 In addition to the standard options described in
-L<the Forks::Queue constructor|"new"/Forks::Queue>, the
+L<the Forks::Queue constructor|Forks::Queue/"new">, the
 C<Forks::Queue::Shmem> constructor also recognizes some
 additional options:
 
